"use strict";
require("isomorphic-fetch");
// The target will be window in a browser and global in node
var target = ((typeof window === "undefined") ? global : window);
// Make sure console.debug exists (it doesn't in node)
if (!target.console.debug)
    target.console.debug = target.console.log;
// Save the original console
var originalConsole = target.console;
// Get information about the platform
var platform = require("platform");
function getLevel(level) {
    switch (level.toLowerCase()) {
        case "log": return 7;
        case "debug": return 7;
        case "info": return 6;
        case "warn": return 4;
        case "error": return 3;
    }
}
function configure(opts) {
    var capitalize = function (s) { return s.length > 0 ? s[0].toUpperCase() + s.substring(1) : ""; };
    function makeLogger(originalFn, level) {
        return function (message) {
            var contexts = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                contexts[_i - 1] = arguments[_i];
            }
            // If the first element of contexts is a string then that is the fullMessage
            var fullMessage = null;
            if (contexts.length > 0 && typeof contexts[0] === "string") {
                fullMessage = contexts.shift();
            }
            // Put all the remaining contexts together into a single object
            var additionalFields = Object.assign.apply(Object, [{}].concat(contexts));
            // Only log to Graylog if level is at least opts.level
            if (getLevel(level) <= getLevel(opts.level || "log")) {
                // If any of the additional fields are 'id' throw an (original) error
                if (Object.keys(additionalFields).filter(function (key) { return key === "id"; }).length === 1) {
                    target.console.originalError("Additional fields named `id` are not allowed by the GELF format");
                }
                else {
                    // Construct the basic gelf message
                    var gelfMessage = {
                        "version": "1.1",
                        "host": opts.host,
                        "short_message": message,
                        "level": getLevel(level),
                    };
                    // If message is an Error then extract useful bits
                    if (Object.prototype.toString.call(message) === "[object Error]") {
                        gelfMessage.short_message = message.message;
                        gelfMessage.full_message = message.stack;
                    }
                    // Add the full_message, if there is one
                    if (fullMessage)
                        gelfMessage.full_message = fullMessage;
                    // Add all the information we got from platform.js
                    gelfMessage.platform = { platform: platform };
                    // Plus the screen size details, if we can get them
                    if (target.screen) {
                        gelfMessage.platform.platform.screen = {
                            availHeight: target.screen.availHeight,
                            availLeft: target.screen.availLeft,
                            availTop: target.screen.availTop,
                            availWidth: target.screen.availWidth,
                            colorDepth: target.screen.colorDepth,
                            height: target.screen.height,
                            orientation: {
                                angle: target.screen.orientation.angle,
                                type: target.screen.orientation.type
                            },
                            pixelDepth: target.screen.pixelDepth,
                            width: target.screen.width
                        };
                    }
                    // Add any static properties passed in at config time
                    for (var key in (opts.staticProperties || {}))
                        gelfMessage[("_" + key)] = opts.staticProperties[key];
                    // Add all the optional parameters, with underscores
                    for (var key in additionalFields)
                        gelfMessage[("_" + key)] = additionalFields[key];
                    // And finally make the request against the Graylog endpoint.  We don't care if it fails.
                    fetch(opts.endpoint, {
                        method: "post",
                        body: JSON.stringify(gelfMessage)
                    });
                }
            }
            // Write to the console
            originalFn.call.apply(originalFn, [target.console, message + (fullMessage ? ": " + fullMessage : "")].concat(contexts));
        };
    }
    // Overwrite the console methods with our wrapped versions (renaming the original)
    ["log", "debug", "info", "warn", "error"].forEach(function (level) {
        target.console[("original" + capitalize(level))] = target.console[level];
        target.console[level] = makeLogger(originalConsole[level], level);
    });
}
exports.configure = configure;
//# sourceMappingURL=index.js.map